{Program dostosowany pod zadanie 3 lista 2.}
program ChaosGameSDL;
uses SDL, matrix, SysUtils;
const
   WIDTH = 640;
   HEIGHT = 480;
var
   screen : PSDL_Surface;
   
   procedure PutPixel (const x, y : Integer; const r,g,b : Byte);
   var
      PixelColor : LongWord;
      PixelLocation : ^LongWord;
   begin
      PixelColor := SDL_MapRGB (screen^.format, r, g, b);
      PixelLocation := screen^.pixels +
         y * screen^.pitch + x * screen^.format^.BytesPerPixel;
      PixelLocation^ := PixelColor;
   end;
   
   procedure DrawPoint (const x, y, r, g, b : Double);
   var PixX, PixY : Integer;
   begin
      Assert ((0 <= r) and (r <= 1) and (0 <= g) and (g <= 1)
              and (0 <= b) and (b <= 1),
              'ChaosGameSDL.DrawPoint: RGB component outside 0..1 range');      
      PixX := trunc (x * WIDTH);
      PixY := HEIGHT - trunc (y * HEIGHT) - 1;
      if (0 <= PixX) and (PixX < WIDTH)
         and (0 <= PixY) and (PixY < HEIGHT)
      then PutPixel (PixX, PixY,
                     round ($ff * r), round ($ff * g), round ($ff * b));
   end;
   
   function OperatorNorm2D (const m : Tmatrix3_double) : Double;
   var t, s : Double;
   begin
      t := m.data[0,0]*m.data[0,0] + m.data[0,1]*m.data[0,1]
         + m.data[1,0]*m.data[1,0] + m.data[1,1]*m.data[1,1];
      s := abs (m.data[0,0]*m.data[1,1] - m.data[0,1]*m.data[1,0]);
      result := 0.5 * (sqrt (t+2*s) + sqrt (t-2*s))
   end;

   procedure RandomFunction (out m : Tmatrix3_double; out prob : Double);
   function Rand : Double; begin Rand := Random * 1.8 - 0.9 end;
   begin
      m.Init_identity;
      repeat
         {Linear part.}
         m.data[0,0] := Rand; m.data[0,1] := Rand;
         m.data[1,0] := Rand; m.data[1,1] := Rand;
         {Translation part.}
         m.data[0,2] := Rand; m.data[1,2] := Rand;
         {Weight of the function.}
         prob := Random
      until OperatorNorm2D (m) < 1
   end;
   
var
   {We use an N+1 dimensional matrix representation
    of affine transformations.}
   IFS_Vectors : array[0..4] of Tmatrix3_double;
   {Probabilities are not normalized, to faciliate random changes.}
   IFS_Probs : array[0..4] of Double;
   r, c, SumProbs : Double;
   epochs, t, branch : Integer;
   point : Tvector3_double;
   FileName : String;
begin
   Randomize;
   for t := 0 to 4 do RandomFunction (IFS_Vectors[t], IFS_Probs[t]);
   
   SumProbs := 0;
   for t := 0 to 4 do SumProbs += IFS_Probs[t];
   
   SDL_Init (SDL_INIT_VIDEO);
   screen := SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_SWSURFACE);   
   
   Write ('How many iterations: ');
   ReadLn (epochs);
   {The third dimension must always be 1 -- it produces translation.}   
   point.init (0, 0, 1);
   c := 1;
   for t := 1 to epochs do
   begin
      r := Random * SumProbs;
      c := 0.7*c + 0.3 * (r / SumProbs);
      branch := 0;
      while (branch < Length (IFS_Probs))
            and (IFS_Probs[branch] < r) do
      begin
         r := r - IFS_Probs[branch];
         Inc (branch)
      end;
      point := IFS_Vectors[branch] * point;
      {Print the point with a fancy color.}
      DrawPoint (point.data[0], point.data[1], c, (c + 1) / 2, 1 - c)
   end;
   
   SDL_Flip (screen);
   Write ('Give name for the image: ');
   ReadLn (FileName);
   SDL_SaveBMP (screen, PChar (FileName+'.bmp'));
   
   SDL_Quit
end.