{Draw a fractal and save it as a BMP image.}
program ChaosGameSDL;
uses SDL, matrix;
const
   WIDTH = 640;
   HEIGHT = 480;
   IFS_VectorsNum = 4;
   IFS_Barnsley_fern_probs : array[0..2] of Real = (0.02, 0.15, 0.13);
   IFS_fern_1: array[0..2,0..2] of double = ((0, 0, 0.5),
                                             (0, 0.27, 0),
                                             (0, 0, 1));
   IFS_fern_2: array[0..2,0..2] of double = ((-0.14, 0.26, 0.57),
                                             (0.25, 0.22, -0.04),
                                             (0, 0, 1));
   IFS_fern_3: array[0..2,0..2] of double = ((0.17, -0.21, 0.41),
                                             (0.22, 0.18, 0.09),
                                             (0, 0, 1));
   IFS_fern_4: array[0..2,0..2] of double = ((0.78, 0.03, 0.11),
                                             (-0.03, 0.74, 0.27),
                                             (0, 0, 1));
var
   screen : PSDL_Surface;
   procedure PutPixel (const x, y : Integer; const r,g,b : Byte);
   var
      PixelColor : LongWord;
      PixelLocation : ^LongWord;
   begin
      PixelColor := SDL_MapRGB (screen^.format, r, g, b);
      PixelLocation := screen^.pixels +
         y * screen^.pitch + x * screen^.format^.BytesPerPixel;
      PixelLocation^ := PixelColor;
   end;
   
   procedure DrawPoint (const x, y, r, g, b : Real);
   begin
      Assert ((0 <= r) and (r <= 1) and (0 <= g) and (g <= 1)
              and (0 <= b) and (b <= 1),
              'ChaosGameSDL.DrawPoint: RGB component outside 0..1 range');      
      if (0 <= x) and (x < 1) and (0 <= y) and (y < 1)
      then PutPixel (trunc (x * WIDTH), HEIGHT - trunc (y * HEIGHT) - 1,
                     round ($ff * r), round ($ff * g), round ($ff * b));
   end;
   
var
   {We use an N+1 dimensional matrix representation
    of affine transformations.}
   IFS_Vectors : array[0..IFS_VectorsNum-1] of Tmatrix3_double;
   {The last vector probability is: 1 - sum of the others.}
   IFS_probs : array[0..IFS_VectorsNum-2] of Real;
   r, c : Real;
   epochs, t, branch : Integer;
   point : Tvector3_double;
   FileName : String;
begin
   IFS_probs := IFS_Barnsley_fern_probs;
   IFS_Vectors[0].data := IFS_fern_1;
   IFS_Vectors[1].data := IFS_fern_2;
   IFS_Vectors[2].data := IFS_fern_3;
   IFS_Vectors[3].data := IFS_fern_4;
   
   SDL_Init (SDL_INIT_VIDEO);
   screen := SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_SWSURFACE);   
   
   Write ('How many iterations: ');
   ReadLn (epochs);
   {The third dimension must always be 1 -- it produces translation.}   
   point.init (0, 0, 1);
   c := 1;
   for t := 1 to epochs do
   begin
      r := random; c := 0.6*c + 0.4*r; branch := 0;
      while (branch < Length (IFS_probs))
            and (IFS_probs[branch] < r) do
      begin
         r := r - IFS_probs[branch];
         Inc (branch)
      end;
      point := IFS_Vectors[branch] * point;
      {Print the point with a fancy color.}
      DrawPoint (point.data[0], point.data[1], c, (c + 1) / 2, 1 - c)
   end;
   
   SDL_Flip (screen);
   Write ('Give name for the image: ');
   ReadLn (FileName);
   SDL_SaveBMP (screen, PChar (FileName+'.bmp'));
   
   SDL_Quit
end.