{Draw a fractal and save it as a BMP image.}
program ChaosGameSDL;
uses SDL, matrix;
const
   WIDTH = 640;
   HEIGHT = 480;
var
   screen : PSDL_Surface;
   procedure PutPixel (const x, y : Integer; const r,g,b : Byte);
   var
      PixelColor : LongWord;
      PixelLocation : ^LongWord;
   begin
      PixelColor := SDL_MapRGB (screen^.format, r, g, b);
      PixelLocation := screen^.pixels +
         y * screen^.pitch + x * screen^.format^.BytesPerPixel;
      PixelLocation^ := PixelColor;
   end;
   procedure DrawPoint (const x, y, r, g, b : Real);
   begin
      Assert ((0 <= r) and (r <= 1) and (0 <= g) and (g <= 1)
              and (0 <= b) and (b <= 1),
              'ChaosGameSDL.DrawPoint: RGB component outside 0..1 range');      
      if (0 <= x) and (x < 1) and (0 <= y) and (y < 1)
      then PutPixel (trunc (x * WIDTH), HEIGHT - trunc (y * HEIGHT) - 1,
                     round ($ff * r), round ($ff * g), round ($ff * b));
   end;
var
   IFS_Sierpinski : array[0..2] of Tmatrix3_double;
   {The last vector probability is: 1 - sum of the others.}
   IFS_Sierpinski_probs : array[0..1] of Real = (1/3, 1/3);
   
   r, c, x, y : Real;
   epochs, t, branch : Integer;
   point : Tvector3_double;
   FileName : String;
begin
   FillByte(IFS_Sierpinski, SizeOf(IFS_Sierpinski), 0);
   IFS_Sierpinski[0].init (1/2, 0, 0,
                           0, 0.5, 0,
                           0, 0, 1);
   IFS_Sierpinski[1].init (1/2, 0, 1/2,
                           0, 1/2, 0,
                           0, 0, 1);
   IFS_Sierpinski[2].init (1/2, 0, 0.5 / 2,
                           0, 1/2, sqrt (3) / 4,
                           0, 0, 1);
   SDL_Init (SDL_INIT_VIDEO);
   screen := SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_SWSURFACE);   
   Write ('How many iterations: ');
   ReadLn (epochs);
   point.init (0, 0, 1);
   c := 1;
   for t := 1 to epochs do
   begin
      r := random; c := 0.8*c + 0.2*r; branch := 0;
      while (branch < Length (IFS_Sierpinski_probs))
            and (IFS_Sierpinski_probs[branch] < r) do
      begin
         r := r - IFS_Sierpinski_probs[branch];
         Inc (branch)
      end;
      point := IFS_Sierpinski[branch] * point;
      x := point.data[0]; y := point.data[1];
      DrawPoint (x, y, c, (c + 1) / 2, 1 - c);
   end;
   SDL_Flip (screen);
   Write ('Give name for the image: ');
   ReadLn (FileName);
   SDL_SaveBMP (screen, PChar (FileName+'.bmp'));
   SDL_Quit
end.