{Percolation of a fluid through a porous material.}
program Percolation;
uses sdl;

function Quitted : Boolean;
var event : TSDL_Event;
begin
   Quitted := False;
   while SDL_PollEvent (@event) = 1
   do case event.type_ of
      SDL_KeyDown:
         Quitted := Quitted or (event.key.keysym.sym = LongWord ('q'));
      SDL_QuitEv: Quitted := True;
   end;
end;
   
var   
   screen : PSDL_Surface;
   
   procedure PutPixel (const x, y : Integer; const r,g,b : Byte);
   var
      PixelColor : LongWord;
      PixelLocation : ^LongWord;
   begin
      PixelColor := SDL_MapRGB (screen^.format, r, g, b);
      PixelLocation := screen^.pixels +
         y * screen^.pitch + x * screen^.format^.BytesPerPixel;
      PixelLocation^ := PixelColor;
   end;
   
type
   Board = array of array of Boolean;
   
   procedure DrawLayer (const layer : Board;
                        const r,g,b : Byte);
   var i, j : Integer;
   begin
      for i := 0 to High (layer) do
         for j := 0 to High (layer[0]) do
            if layer[i,j] then PutPixel (i, j, r,g,b)
   end;
   
var   
   open : Board;
   full : Board;
   
   function Percolated : Boolean;
   var i, depth : Integer;
   begin
      depth := High (full[0]);
      Percolated := False;
      for i := 0 to High (full) do
         Percolated := Percolated or full[i, depth]
   end;
   
var
   width, depth : Integer;
   
   procedure Flow (const i,j : Integer);
   begin
      if (i >= 0) and (i < width) and (j >= 0) and (j < depth)
         and open[i,j] and not full[i,j]
      then begin
         full[i,j] := True;
         Flow (i+1,j); {First downwards: greedy.}
         Flow (i,j+1);
         Flow (i-1,j);
         Flow (i,j-1)
      end
   end;
   
var
   i, j : Integer;
   prob : Real;
   trials : Integer = 0;
   successes : Integer = 0;
begin
   WriteLn ('Percolation.');
   Write ('width of material = '); ReadLn (width);
   Write ('depth of material = '); ReadLn (depth);
   SetLength (open, width, depth);
   SetLength (full, width, depth);
   Write ('probability of pores (cool number is 0.593) = ');
   ReadLn (prob);
   
   SDL_Init(SDL_INIT_VIDEO);
   screen := SDL_SetVideoMode(width, depth, 8,
                              SDL_SWSURFACE or SDL_DOUBLEBUF);
   repeat
      for i := 0 to width - 1 do
         for j := 0 to depth - 1 do
         begin
            full[i,j] := False;
            open[i,j] := random < prob
         end;
      for i := 0 to width - 1 do
         Flow (i, 0);
      Inc (trials); if Percolated then Inc (successes);
      DrawLayer (open, $ff, $ff, 0);
      DrawLayer (full, 0, 0, $ff);
      
      SDL_Flip (screen);
      SDL_Delay (20)
   until Quitted;
   
   SDL_FreeSurface(screen);
   SDL_Quit;
   WriteLn ('Trials: ', trials, '; percolation probability: ',
            successes / trials :0:3)
end.